
%ECUACION_ONDAS_2D Ecuacin de ondas en 2 dimensiones
%
% [U,X,Y,T] = ECUACION_CALOR_2D(xmin,xmax,dx,ymin,ymax,dy,tmax,dt,alfa,U0) calcula la 
%   ecuacin del calor 2D U(x,y,t) para los lmites x,y especificados con sus deltas
%   de t = 0 a tmax con pasos de dt. U0 es una matriz acorde a las dimensiones de los
%   ejes x,y que describe el valor de U para t0
%
function [U,X,Y,T] = ecuacion_calor_2d(xmin,xmax,dx,ymin,ymax,dy,tmax,dt,alfa,U0)
    %Nos aseguramos de que se cumplen las precondiciones
    assert(xmin<xmax,'xmin tiene que ser menor que xmax');
    assert(ymin<ymax,'ymin tiene que ser menor que ymax');
    assert(dx>0,'dx tiene que ser positivo');
    assert(dy>0,'dy tiene que ser positivo');
    assert(dt>0,'dt tiene que ser positivo');
    assert(tmax>0,'tmax tiene que ser positivo');
   
    X = xmin:dx:xmax;
    Y = ymin:dy:ymax;
    T = 0:dt:tmax;
    
    %comprobamos dimensiones de vectores y matrices
    assert(length(X)>1,'intervalo en x muy pequeo o dx muy grande');
    assert(length(Y)>1,'intervalo en y muy pequeo o dy muy grande');
    assert(length(T)>1,'tmax muy pequeo o dt muy grande');
    assert(size(U0,1) == length(X),'el numero de filas de U0 tiene que ser igual a la dimensin de X');
    assert(size(U0,2) == length(Y),'el numero de columnas de U0 tiene que ser igual a la dimensin de Y');


    %preparamos la funcin con los valores iniciales y los valores 
    %de los bordes a cero.
    U = zeros(length(X)+2,length(Y)+2,length(T));
    U(2:length(X)+1,2:length(Y)+1,1) = U0;

    %calculamos los valores de U para t_2 en adelante
    for t  = 2:length(T)
        %calculamos cada Uij para este t
        for i = 2:length(X)+1
            for j = 2:length(Y)+1
                parcialx = (U(i-1,j,t-1) - 2*U(i,j,t-1) +U(i+1,j,t-1))/(dx^2);
                parcialy = (U(i,j-1,t-1) - 2*U(i,j,t-1) +U(i,j+1,t-1))/(dy^2);
                U(i,j,t) = alfa*(parcialx+parcialy)*dt + U(i,j,t-1);
            end
        end
    end
    %quitamos el borde auxiliar de la superfie
    U = U(2:length(X)+1,2:length(Y)+1,:);
return;

